open Econv;;
open Epoly;;
open Efile;;

exception Exit;;
exception Found of (int option);;

let print x =
 try (
  let y = parse x in
  Printf.printf ": %s\t-> " (string_of_tree y);
  let z = reduce y in
  Printf.printf "%s\n" (string_of_poly z))
 with
   Parse s ->     Printf.printf ": %s\t-> Parsing failure: %s\n" x s
  |Reduction s -> Printf.printf "Reduction failure: %s\n" s;;

let commands = [| [|""; "exit"; "quit"|] ; [|"help"|] ; [|"exec" ; "read" ; "open"|] |];;
let descrs = [| "terminate execution" ; "display help" ; "open file and parse every line of it" |];;

let find_command s =
 try (
  for i = 0 to Array.length commands - 1 do
   for j = 0 to Array.length commands.(i) - 1 do
    if s = commands.(i).(j) then raise (Found (Some i));
   done;
  done; raise (Found None))
 with
   Found x -> x;;

let exit s = raise Exit;;

let help s =
 if Array.length s = 1 then (
 print_endline "   Available commands:";
 for i = 0 to Array.length commands - 1 do
  for j = 0 to Array.length commands.(i) - 1 do
   Printf.printf " | '%s'" commands.(i).(j);
  done;
  Printf.printf " :\t%s\n" descrs.(i);
 done) else (
  match find_command s.(1) with
    None -> print_endline (s.(1) ^ " : command not found")
   |Some x -> print_endline (descrs.(x)));;

let rec execute s =
 let parse_file s =
  List.iter (fun s -> if execute s then print s) (list_of_file s.(1)) in
 let actions = [| exit ; help ; parse_file |] in (* the array of actions assigned to specific commands *)
 let s = args_of_string s in
 match find_command s.(0) with
   None -> true
  |Some x -> actions.(x) s; false;;


let rec interface () =
 print_string "# ";
 let s = read_line () in
 if execute s then print s;
 interface ();;

print_endline "evaluator version 0.2 (c) 2010 lcme.ucoz.ru";;
try interface () with Exit -> ();;